<?php declare(strict_types=1);

namespace EuKit\Base;

use Infomaker\Everyware\Base\ArticleBodyParser;
use Infomaker\Everyware\NewsML\NewsMLTransformerManager;
use Infomaker\Everyware\Support\Collection;
use Infomaker\Everyware\Support\Date;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Support\NewRelicLog;
use OcArticle;

/**
 * Class NewsMLArticle
 * @property string     headline
 * @property array      channels
 * @property Date       pubdate
 *
 * @property Collection articles
 * @property Collection authors
 * @property Collection categories
 * @property Collection places
 * @property Collection stories
 * @property Collection tags
 *
 * @package EuKit\Base
 */
class NewsMLArticle extends GenericPropertyObject
{
    public function __construct(PropertyObject $object)
    {
        $this->fill($object->getProperties());
    }

    /**
     * Contains an instance of the parser for the article
     *
     * @var ArticleBodyParser
     */
    protected $parser;

    public function getParsedBody(): ArticleBodyParser
    {
        return $this->getBodyParser();
    }

    public function getBodyParser(): ArticleBodyParser
    {
        if ($this->parser === null) {
            $this->parser = new ArticleBodyParser($this);
        }

        return $this->parser;
    }

    public function getNewsMLTeaser()
    {
        $newsml = $this->get('teaser_raw');

        if (empty($newsml)) {
            NewRelicLog::error(sprintf('Will not parse empty value from Object: %s',
                $this->get('uuid')));
            return [];
        }

        $transformer = NewsMLTransformerManager::createObjectTransformer($newsml);

        $result = $transformer->transform($newsml);

        return count($result) > 0 ? array_shift($result) : null;
    }

    public function hasRelatedArticles(): bool
    {
        return $this->articles->isNotEmpty();
    }

    public function getCategories(): Collection
    {
        return Collection::make($this->categories)->unique('name');
    }

    public function getTags(): Collection
    {
        return Collection::make($this->tags)->unique('name');
    }

    public function getArticleTags(): Collection
    {
        return $this->getFilteredPlaces()->merge($this->getTags())->unique('name')->sortBy('name');
    }

    public function getAuthors(): Collection
    {
        return Collection::make($this->authors);
    }

    public function getStories(): Collection
    {
        return Collection::make($this->stories)->unique('name');
    }

    public function getChannels(): Collection
    {
        return Collection::make($this->channels);
    }

    public function getPlaces(): Collection
    {
        return Collection::make($this->places)->unique('name');
    }

    /**
     * Retrieve a list of Place concepts without the hierarchical relations
     *
     * @since 1.0.0
     * @return Collection
     */
    public function getFilteredPlaces(): Collection
    {
        $places = $this->getPlaces();
        $placeRelationUuids = array_column($places->toArray(), 'parent_uuid', 'uuid');

        $placesToKeep = $placeRelationUuids;

        foreach ($placeRelationUuids as $uuid => $parent_uuid) {
            // Remove my parent
            unset($placesToKeep[$parent_uuid]);
        }

        return $places->whereIn('uuid', array_keys($placesToKeep));
    }

    /**
     * Retrieve related article by uuid
     *
     * @param string $uuid
     *
     * @return static|null
     */
    public function getRelatedArticle(string $uuid)
    {
        $articles = $this->relatedArticles()->whereIn('uuid', $uuid);

        return $articles->isNotEmpty() ? $articles->first() : null;
    }

    public function relatedArticles(): Collection
    {
        return Collection::make($this->articles);
    }

    public function getPubDate(): Date
    {
        return Date::parse($this->pubdate);
    }

    public static function createFromOcArticle(OcArticle $article)
    {
        $propertyObject = new GenericPropertyObject();
        $propertyObject->fill($article->get_all_properties());

        $propertyObject->fill([
            'uuid' => $article->get_value('uuid'),
            'headline' => $article->get_value('headline'),
            'pubdate' => $article->get_value('pubdate'),
            'updated' => $article->get_value('updated'),
            'body_raw' => $article->get_value('bodyraw'),
            'teaser_raw' => $article->get_value('teaserraw'),
            'Authors' => $article->get_value('Authors'),
            'leadin' => $article->get_value('leadin'),
            'permalink' => $article->get_permalink(),
        ]);

        return new static($propertyObject);
    }
}
